#!/bin/sh
#===============================================================
# Hitachi Server Navigator, Sub Installer for RHEL 
#===============================================================
# HISTORY
# Rev. 1.00             Hitachi       New Release
# Rev. 4.7.0            Hitachi       Support new HW
#                change function : do_check_install_powermonitor
#===============================================================
# All Rights Reserved, Copyright (C) 2015 Hitachi, Ltd.
#===============================================================


#-<private>-----------------------------------------------------
# Define the component, files and path.
#---------------------------------------------------------------
OLDSNV_PACKAGE_NAME=snv
CLI_PACKAGE_NAME=snvcli
UM_PACKAGE_NAME=snvupdtmngr
PCL_PACKAGE_NAME=snvpowermonitor
OS_TYPE=el

INSTCONF=hitachi.ini
CMDNAME=$SNV_PARENT_SHELL
RPMPATH=`dirname $0`
SNVPATH=/opt/hitachi/snv
LICENSEFILE=/opt/hitachi/snv/cli/license.txt
DLSETTINGFILE=/opt/hitachi/snv/um/DownloadInfo.ini
BACKUP_TEMP=/opt/hitachi/snvtmp
MINOR_ROLLBACK=0
SNV_RESULT=0
SNV_TRUE=1
SNV_FALSE=0

SNV_UM="updtmngr"
SNV_SOM="somservice"
SNV_PCL="snvpowermonitor"

IS_DOMESTIC="true"

NON_SELECTED_PACKAGE=0

RHVER=

#-<private>-----------------------------------------------------
# Version information of the installed package.
#---------------------------------------------------------------
INSTALLED_CLI_MAJOR_VER=0
INSTALLED_CLI_RELEASE_VER=0
INSTALLED_UM_MAJOR_VER=0
INSTALLED_UM_RELEASE_VER=0
INSTALLED_PCL_MAJOR_VER=0
INSTALLED_PCL_RELEASE_VER=0
INSTALLED_OLDSNV_MAJOR_VER=0
INSTALLED_OLD_RELEASE_VER=0

#-<private>-----------------------------------------------------
# Flag for the installation possible judgment of the package.
#---------------------------------------------------------------
TARGET_PACKAGE_CLI=1
TARGET_PACKAGE_UM=1
TARGET_PACKAGE_PCL=1

#=<public>======================================================
# Show usage.
#===============================================================
function do_usage() {
	echo "Usage:"
	echo "  $CMDNAME [option]"

	if [ "$IS_DOMESTIC" = "true" ]; then
		do_check_install_powermonitor
		
		if [ $TARGET_PACKAGE_PCL -eq 0 ]; then
			echo "    -i [snvcli | updtmngr]  install all system or optional package"
			echo "    -o file                 install with oem file"
			echo "    -e [snvcli | updtmngr]  uninstall"
			echo "    -l                      show installed package detail"
			echo "       --help               show help message"
			echo "       --version            show installer version"
		else
			echo "    -i [snvcli | updtmngr | snvpowermonitor]  install all system or optional package"
			echo "    -o file                                   install with oem file"
			echo "    -e [snvcli | updtmngr | snvpowermonitor]  uninstall"
			echo "    -l                                        show installed package detail"
			echo "       --help                                 show help message"
			echo "       --version                              show installer version"
		fi
	else
		echo "    -i                 install all"
		echo "    -o file            install with oem file"
		echo "    -e                 uninstall"
		echo "    -l                 show installed package detail"
		echo "       --help          show help message"
		echo "       --version       show installer version"
	fi
	
	exit 1
}

#=<public>======================================================
# Wait for a process end.
#===============================================================
function do_wait() {
	## not use
	return 0
}

#=<private>=====================================================
# Acquire version information from an installed package.
#  @param  $1 package name
#  @return 0 Success
#  @return 1 A package is not installed
#===============================================================
function get_version_if_installed() {
	PKG_NAME=$1
	rpm -q $PKG_NAME >>$SNV_LOGFILE 2>>$SNV_LOGFILE
	if [ $? -eq 0 ] ; then
		rpm -qi $PKG_NAME >>$SNV_LOGFILE 2>>$SNV_LOGFILE
		cur_ver=`rpm -q $PKG_NAME | cut -d"-" -f2 | head -1`
		MAJOR_VER=`echo $cur_ver | cut -d"." -f2`
		RELEASE_VER=`rpm -q $PKG_NAME | cut -d"-" -f3 | sed -e 's/\./ /g' | awk '{print $1"."$2}'`
		do_putlog "installed $PKG_NAME MAJOR[$MAJOR_VER], RELEASE[$RELEASE_VER]"
		SNV_RESULT=0
	else
		SNV_RESULT=1
	fi
}

#=<private>=====================================================
# Acquire the version information of each package collectively.
# (When be not installed, MAJOR_VER, RELEASE_VER becomes 0)
#===============================================================
function get_installed_package_versions() {
	do_putlog "check installed package."

	get_version_if_installed $CLI_PACKAGE_NAME
	if [ $SNV_RESULT -eq 0 ]; then
		INSTALLED_CLI_MAJOR_VER=$MAJOR_VER
		INSTALLED_CLI_RELEASE_VER=$RELEASE_VER
	fi

	get_version_if_installed $UM_PACKAGE_NAME
	if [ $SNV_RESULT -eq 0 ]; then
		INSTALLED_UM_MAJOR_VER=$MAJOR_VER
		INSTALLED_UM_RELEASE_VER=$RELEASE_VER
	fi

	get_version_if_installed $PCL_PACKAGE_NAME
	if [ $SNV_RESULT -eq 0 ]; then
		INSTALLED_PCL_MAJOR_VER=$MAJOR_VER
		INSTALLED_PCL_RELEASE_VER=$RELEASE_VER
	fi

	get_version_if_installed $OLDSNV_PACKAGE_NAME
	if [ $SNV_RESULT -eq 0 ]; then
		INSTALLED_OLDSNV_MAJOR_VER=$MAJOR_VER
		INSTALLED_OLD_RELEASE_VER=$RELEASE_VER
	fi
}

#=<private>=====================================================
# Set a package targeted for installation
#   @param $1 snvcli install or not.
#   @param $2 updtmngr install or not.
#   @param $3 powerlog install or not.
#===============================================================
function set_target_package() {
	TARGET_PACKAGE_CLI=$1
	TARGET_PACKAGE_UM=$2
	TARGET_PACKAGE_PCL=$3
}

#=<private>=====================================================
# Stop all services.
#===============================================================
function do_service_stop() {
	if [ $TARGET_PACKAGE_UM -eq 1 ] ; then
		if [ $INSTALLED_UM_MAJOR_VER -ne 0 -o $INSTALLED_OLDSNV_MAJOR_VER -ne 0 ]; then
			# UpdateManager stop wait.
			echo "Stop the $SNV_UM"
			do_putlog "Stop the Update ManagerService."
			do_wait $SNV_UM 10
			if [ $RHVER -lt 7 ];then
				/sbin/service $SNV_UM stop >>$SNV_LOGFILE 2>>$SNV_LOGFILE
			else
				/usr/bin/systemctl stop $SNV_UM >>$SNV_LOGFILE 2>>$SNV_LOGFILE
			fi
		fi
	fi

	if [ $TARGET_PACKAGE_PCL -eq 1 ] ; then
		if [ $INSTALLED_PCL_MAJOR_VER -ne 0 ]; then
			# stopping SnvPowerMonitor
			echo "Stop the $SNV_PCL"
			do_putlog "Stop the SnvPowerMonitor."
			do_wait $SNV_PCL 10
			if [ $RHVER -lt 7 ];then
				/sbin/service $SNV_PCL stop >>$SNV_LOGFILE 2>>$SNV_LOGFILE
			else
				/usr/bin/systemctl stop $SNV_PCL >>$SNV_LOGFILE 2>>$SNV_LOGFILE
			fi
		fi
	fi

	if [ $TARGET_PACKAGE_CLI -eq 1 ] ; then
		if [ $INSTALLED_CLI_MAJOR_VER -ne 0 -o $INSTALLED_OLDSNV_MAJOR_VER -ne 0 ]; then
			# SOM stop wait.
			echo "Stop the $SNV_SOM"
			do_putlog "Stop the SOM Service."
			do_wait $SNV_SOM 10
			if [ $RHVER -lt 7 ];then
				/sbin/service $SNV_SOM stop >>$SNV_LOGFILE 2>>$SNV_LOGFILE
			else
				/usr/bin/systemctl stop $SNV_SOM >>$SNV_LOGFILE 2>>$SNV_LOGFILE
			fi
		fi
	fi
}

#=<private>=====================================================
# Start all services.
#===============================================================
function do_service_start() {
	if [ $TARGET_PACKAGE_CLI -eq 1 ] ; then
		echo "Start the $SNV_SOM"
		do_putlog "Start the SOM Service."
		if [ $RHVER -lt 7 ];then
			/sbin/service $SNV_SOM start >>$SNV_LOGFILE 2>>$SNV_LOGFILE
		else
			/usr/bin/systemctl start $SNV_SOM >>$SNV_LOGFILE 2>>$SNV_LOGFILE
		fi
	fi

	if [ $TARGET_PACKAGE_UM -eq 1 ] ; then
		echo "Start the $SNV_UM"
		do_putlog "Start the Update Manager Service."
		if [ $RHVER -lt 7 ];then
			/sbin/service $SNV_UM start >>$SNV_LOGFILE 2>>$SNV_LOGFILE
		else
			/usr/bin/systemctl start $SNV_UM >>$SNV_LOGFILE 2>>$SNV_LOGFILE
		fi
	fi

	if [ $TARGET_PACKAGE_PCL -eq 1 ] ; then
		echo "Start the $SNV_PCL"
		do_putlog "Start the SnvPowerMonitor Service."
		if [ $RHVER -lt 7 ];then
			/sbin/service $SNV_PCL start >>$SNV_LOGFILE 2>>$SNV_LOGFILE
		else
			/usr/bin/systemctl start $SNV_PCL >>$SNV_LOGFILE 2>>$SNV_LOGFILE
		fi
	fi
}

#=<private>=====================================================
# Confirm the existence of the installation RPM file.
#   When a file does not exist, output an error, and be finished.
#===============================================================
function check_rpmfiles_exist() {
	if [ $TARGET_PACKAGE_CLI -eq 1 -a ! -e "${RPMPATH}/${CLI_PACKAGE_NAME}-${SNV_CLI_VERSION}.${OS_TYPE}.${SNV_CPU_ARCH}.rpm" ]; then
		echo -e "Failed in installation.\nNot found ${CLI_PACKAGE_NAME}-${SNV_CLI_VERSION}.${OS_TYPE}.${SNV_CPU_ARCH}.rpm file."
		do_putlog "${CLI_PACKAGE_NAME} package is nothing in installer."
		exit 1
	fi
	if [ $TARGET_PACKAGE_UM -eq 1 -a ! -e "${RPMPATH}/${UM_PACKAGE_NAME}-${SNV_UM_VERSION}.${OS_TYPE}.${SNV_CPU_ARCH}.rpm" ]; then
		echo -e "Failed in installation.\nNot found ${UM_PACKAGE_NAME}-${SNV_UM_VERSION}.${OS_TYPE}.${SNV_CPU_ARCH}.rpm file."
		do_putlog "${UM_PACKAGE_NAME} package is nothing in installer."
		exit 1
	fi
	if [ $TARGET_PACKAGE_PCL -eq 1 -a ! -e "${RPMPATH}/${PCL_PACKAGE_NAME}-${SNV_PCL_VERSION}.${OS_TYPE}.${SNV_CPU_ARCH}.rpm" ]; then
		echo -e "Failed in installation.\nNot found ${PCL_PACKAGE_NAME}-${SNV_PCL_VERSION}.${OS_TYPE}.${SNV_CPU_ARCH}.rpm file."
		do_putlog "${PCL_PACKAGE_NAME} package is nothing in installer."
		exit 1
	fi
}

#=<private>=====================================================
# Display the version of the designated package.
#   When a designated package is not installed, do nothing.
#   @param $1 package name
#===============================================================
function show_package_ver() {
	rpm -q $1 >/dev/null 2>&1
	if [ $? -eq 0 ] ; then
		echo " "
		rpm -qi $1
	fi
}

#=<private>=====================================================
# Install a designated package.
#   @param $1 package name    (ex. CLI_PACKAGE_NAME)
#   @param $2 package version (ex. SNV_CLI_VERSION)
#===============================================================
function do_rpm_install() {
	PKG_NAME=$1
	PKG_VERSION=$2

 	do_putlog "install ${RPMPATH}/${PKG_NAME}-${PKG_VERSION}.${OS_TYPE}.${SNV_CPU_ARCH}.rpm"
 	if [ $MINOR_ROLLBACK -eq $SNV_TRUE ]; then
 		rpm -e --allmatches --nodeps $PKG_NAME >>$SNV_LOGFILE 2>>$SNV_LOGFILE
 	fi
	rpm -Uhv --nodeps ${RPMPATH}/${PKG_NAME}-${PKG_VERSION}.${OS_TYPE}.${SNV_CPU_ARCH}.rpm >>$SNV_LOGFILE 2>>$SNV_LOGFILE
	if [ $? -ne 0 ] ; then
		do_putlog "install error. try force install."
		rpm -ihv -oldpackage  -nodeps -force ${RPMPATH}/${PKG_NAME}-${PKG_VERSION}.${OS_TYPE}.${SNV_CPU_ARCH}.rpm >>$SNV_LOGFILE 2>>$SNV_LOGFILE
		if [ $? -ne 0 ] ; then
			echo -e 'The installation failed.\n(Please eject the CDROM.)'
			do_putlog "The installation failed."
			exit 1
		fi
	fi
}

#=<private>=====================================================
# Check already installed.
#   But when snvcli is not targeted for installation, output an
#   error by individual installation, and be finished.
#   @return SNV_TRUE  Installed package not exists
#   @return SNV_FALSE Installed package already exists
#===============================================================
function is_new_installation() {
	if [ $INSTALLED_CLI_MAJOR_VER -eq 0 -a $INSTALLED_UM_MAJOR_VER -eq 0 -a \
		 $INSTALLED_PCL_MAJOR_VER -eq 0 -a $INSTALLED_OLDSNV_MAJOR_VER -eq 0 ]; then
		do_putlog "new install process."
		if [ $TARGET_PACKAGE_CLI -eq 0 ]; then
			echo -e 'Failed in installation.\nA necessary component is not installed.'
			do_putlog "error. not select snvcli"
			exit 1
		fi
		SNV_RESULT=1
	else
		SNV_RESULT=0
	fi
}

#=<private>=====================================================
# Check the old package installed (Before V3.1)
#   @return SNV_TRUE  Old package not installed
#   @return SNV_FALSE Old package is exists
#===============================================================
function is_old_snv_installed() {
	if [ $INSTALLED_OLDSNV_MAJOR_VER -ne 0 ]; then
		TARGET_PACKAGE_CLI=1
		TARGET_PACKAGE_UM=1
		TARGET_PACKAGE_PCL=0
		SNV_RESULT=1
	else
		SNV_RESULT=0
	fi
}

#=<private>=====================================================
# Check the same version is installed.
#===============================================================
function check_already_installed() {
	if [ $INSTALLED_PCL_MAJOR_VER -eq $SNV_PCL_MAJOR_VER -a "$INSTALLED_PCL_RELEASE_VER" = "$SNV_PCL_RELEASE_VER" ]; then
		do_putlog "$PCL_PACKAGE_NAME is same version."
		TARGET_PACKAGE_PCL=0
	fi
	if [ $INSTALLED_UM_MAJOR_VER -eq $SNV_UM_MAJOR_VER -a "$INSTALLED_UM_RELEASE_VER" = "$SNV_UM_RELEASE_VER" ]; then
		do_putlog "$UM_PACKAGE_NAME is same version."
		TARGET_PACKAGE_UM=0
	fi
	if [ $INSTALLED_CLI_MAJOR_VER -eq $SNV_CLI_MAJOR_VER -a "$INSTALLED_CLI_RELEASE_VER" = "$SNV_CLI_RELEASE_VER" ]; then
		do_putlog "$CLI_PACKAGE_NAME is same version."
		TARGET_PACKAGE_CLI=0
	fi

	if [ $TARGET_PACKAGE_CLI -eq 0 -a $TARGET_PACKAGE_UM -eq 0 -a $TARGET_PACKAGE_PCL -eq 0 ]; then
		do_putlog "already installed same version."
		echo "This system already installed same version."
		exit 0
	fi
}

#=<private>=====================================================
# Check dependencies of the automatic updating.
#===============================================================
function check_update_install() {
	
	if [ $INSTALLED_CLI_MAJOR_VER -ne 0 -a $TARGET_PACKAGE_CLI -eq 0 ]; then
		if [ $INSTALLED_CLI_MAJOR_VER -eq $SNV_CLI_MAJOR_VER -a "$INSTALLED_CLI_RELEASE_VER" = "$SNV_CLI_RELEASE_VER" ]; then
			# same version
			TARGET_PACKAGE_CLI=0
		else
			TARGET_PACKAGE_CLI=1
			do_putlog "$CLI_PACKAGE_NAME is auto update."
		fi
	elif [ $INSTALLED_CLI_MAJOR_VER -eq 0 -a $TARGET_PACKAGE_CLI -eq 1 -a $NON_SELECTED_PACKAGE -eq 1 ]; then
		TARGET_PACKAGE_CLI=0
		do_putlog "$CLI_PACKAGE_NAME is not additional install."
	fi

	if [ $INSTALLED_UM_MAJOR_VER -ne 0 -a $TARGET_PACKAGE_UM -eq 0 ]; then
		if [ $INSTALLED_UM_MAJOR_VER -eq $SNV_UM_MAJOR_VER -a "$INSTALLED_UM_RELEASE_VER" = "$SNV_UM_RELEASE_VER" ]; then
			# same version
			TARGET_PACKAGE_UM=0
		else
			TARGET_PACKAGE_UM=1
			do_putlog "$UM_PACKAGE_NAME is auto update."
		fi
	elif [ $INSTALLED_UM_MAJOR_VER -eq 0 -a $TARGET_PACKAGE_UM -eq 1 -a $NON_SELECTED_PACKAGE -eq 1 ]; then
		TARGET_PACKAGE_UM=0
		do_putlog "$UM_PACKAGE_NAME is not additional install."
	fi

	if [ $INSTALLED_PCL_MAJOR_VER -ne 0 -a $TARGET_PACKAGE_PCL -eq 0 ]; then
		if [ $INSTALLED_PCL_MAJOR_VER -eq $SNV_PCL_MAJOR_VER -a "$INSTALLED_PCL_RELEASE_VER" = "$SNV_PCL_RELEASE_VER" ]; then
			# same version
			TARGET_PACKAGE_PCL=0
		else
			TARGET_PACKAGE_PCL=1
			do_putlog "$PCL_PACKAGE_NAME is auto update."
		fi
	elif [ $INSTALLED_PCL_MAJOR_VER -eq 0 -a $TARGET_PACKAGE_PCL -eq 1 -a $NON_SELECTED_PACKAGE -eq 1 ]; then
		TARGET_PACKAGE_PCL=0
		do_putlog "$PCL_PACKAGE_NAME is not additional install."
	fi

}

#=<private>=====================================================
# Compare the minor version.
#===============================================================
function is_rollback_minor() {
	SRC_MINOR_U=`echo $1 | sed -s 's/\./ /g' | awk '{print $1}'`
	SRC_MINOR_L=`echo $1 | sed -s 's/\./ /g' | awk '{print $2}'`
	DST_MINOR_U=`echo $2 | sed -s 's/\./ /g' | awk '{print $1}'`
	DST_MINOR_L=`echo $2 | sed -s 's/\./ /g' | awk '{print $2}'`
	if [ $SRC_MINOR_U -gt $DST_MINOR_U -o $SRC_MINOR_L -gt $DST_MINOR_L ]; then
		SNV_RESULT=1
	else
		SNV_RESULT=0
	fi
}

#=<private>=====================================================
# Check the rollback violation.
#===============================================================
function check_rollback_install() {
	IS_INVALID=0
	if [ $TARGET_PACKAGE_CLI -eq 1 ]; then
		if [ $INSTALLED_CLI_MAJOR_VER -gt $SNV_CLI_MAJOR_VER ]; then
			IS_INVALID=1
		elif [ $INSTALLED_CLI_MAJOR_VER -eq $SNV_CLI_MAJOR_VER ]; then
			is_rollback_minor $INSTALLED_CLI_RELEASE_VER $SNV_CLI_RELEASE_VER
			if [ $SNV_RESULT -eq $SNV_TRUE ]; then
				MINOR_ROLLBACK=1
			fi
		fi
	fi
	if [ $TARGET_PACKAGE_UM -eq 1 ]; then
		if [ $INSTALLED_UM_MAJOR_VER -gt $SNV_UM_MAJOR_VER ]; then
			IS_INVALID=1
		elif [ $INSTALLED_UM_MAJOR_VER -eq $SNV_UM_MAJOR_VER ]; then
			is_rollback_minor $INSTALLED_UM_RELEASE_VER $SNV_UM_RELEASE_VER
			if [ $SNV_RESULT -eq $SNV_TRUE ]; then
				MINOR_ROLLBACK=1
			fi
		fi
	fi
	if [ $TARGET_PACKAGE_PCL -eq 1 ]; then
		if [ $INSTALLED_PCL_MAJOR_VER -gt $SNV_PCL_MAJOR_VER ]; then
			IS_INVALID=1
		elif [ $INSTALLED_PCL_MAJOR_VER -eq $SNV_PCL_MAJOR_VER ]; then
			is_rollback_minor $INSTALLED_PCL_RELEASE_VER $SNV_PCL_RELEASE_VER
			if [ $SNV_RESULT -eq $SNV_TRUE ]; then
				MINOR_ROLLBACK=1
			fi
		fi
	fi
	if [ $IS_INVALID -eq 1 ]; then
		do_putlog "rollback install major number failed"
		echo "The setup failed. (Please eject the CDROM.)"
		echo "A later version of the product is already installed "
		echo "on your computer. "
		echo "The setup cannot continue."
		exit 1
	fi
}

#=<private>=====================================================
# Back up configuration files.
#===============================================================
function do_backup_files() {
	do_putlog "OldSnvVersion:${INSTALLED_OLDSNV_MAJOR_VER} to New Version"
	# env file backup.
	do_putlog "old file backup start."
	mkdir $BACKUP_TEMP >>$SNV_LOGFILE 2>>$SNV_LOGFILE
	mkdir $BACKUP_TEMP/um >>$SNV_LOGFILE 2>>$SNV_LOGFILE
	cp -af $SNVPATH/um/module $BACKUP_TEMP/um/module >>$SNV_LOGFILE 2>>$SNV_LOGFILE
	rm -rf $SNVPATH/um/module >>$SNV_LOGFILE 2>>$SNV_LOGFILE
	cp -af $SNVPATH/um/UpdMgrInfo $BACKUP_TEMP/um/UpdMgrInfo >>$SNV_LOGFILE 2>>$SNV_LOGFILE
	rm -rf $SNVPATH/um/UpdMgrInfo >>$SNV_LOGFILE 2>>$SNV_LOGFILE
	cp -af $SNVPATH/um/DownloadInfo.ini $BACKUP_TEMP/um/DownloadInfo.ini >>$SNV_LOGFILE 2>>$SNV_LOGFILE
	cp -af $SNVPATH/um/UpdateInfo.ini $BACKUP_TEMP/um/UpdateInfo.ini >>$SNV_LOGFILE 2>>$SNV_LOGFILE
	mkdir $BACKUP_TEMP/provider >>$SNV_LOGFILE 2>>$SNV_LOGFILE
	cp -af $SNVPATH/provider/*Rep.ini $BACKUP_TEMP/provider/ >>$SNV_LOGFILE 2>>$SNV_LOGFILE
	cp -af $SNVPATH/log $BACKUP_TEMP/log >>$SNV_LOGFILE 2>>$SNV_LOGFILE
	do_putlog "old file backup end."
	# uninnstall old package
	do_putlog "uninnstall old package."
	rpm -e --allmatches $OLDSNV_PACKAGE_NAME >>$SNV_LOGFILE 2>>$SNV_LOGFILE
	if [ $? -ne 0 ] ; then
		echo -e 'The installation failed.\n(Please eject the CDROM.)'
		do_putlog "old snv uninstallation failed."
		exit 1
	fi
}

#=<private>=====================================================
# Restore configuration files.
#===============================================================
function do_restore_files() {
	# env file restore.
	do_putlog "old file restore start."
	cp -af $BACKUP_TEMP/um/module $SNVPATH/um >>$SNV_LOGFILE 2>>$SNV_LOGFILE
	cp -af $BACKUP_TEMP/um/UpdMgrInfo $SNVPATH/um/UpdMgrInfo >>$SNV_LOGFILE 2>>$SNV_LOGFILE
	cp -af $BACKUP_TEMP/um/DownloadInfo.ini $SNVPATH/um/DownloadInfo.ini >>$SNV_LOGFILE 2>>$SNV_LOGFILE
	cp -af $BACKUP_TEMP/um/UpdateInfo.ini $SNVPATH/um/UpdateInfo.ini >>$SNV_LOGFILE 2>>$SNV_LOGFILE
	cp -af $BACKUP_TEMP/provider/*Rep.ini $SNVPATH/provider/ >>$SNV_LOGFILE 2>>$SNV_LOGFILE
	cp -af $BACKUP_TEMP/log $SNVPATH >>$SNV_LOGFILE 2>>$SNV_LOGFILE
	rm -rf $BACKUP_TEMP >>$SNV_LOGFILE 2>>$SNV_LOGFILE
	do_putlog "old file restore end."
}

#=<private>=====================================================
# Check the license file.
#   @return SNV_TRUE  Success
#   @return SNV_FALSE Error
#===============================================================
function do_setup_config() {
	CONFPROCESS=0
	
	if [ "$IS_DOMESTIC" = "true" ]; then
		if [ -e $INSTCONF ];then
			CFILE=`grep "file=" $INSTCONF | sed 's/=/ /g' | awk '{ print $2 }'`
			CFILE=`echo $CFILE | sed -e s/\n\r//`
			WURL=`grep "url=" $INSTCONF | sed 's/=/ /g' | awk '{ print $2 }'`

			if [ "$CFILE" != "" -a -e "$RPMPATH/$CFILE" ];then
				cp -f $RPMPATH/$CFILE $LICENSEFILE
				chmod 644 $LICENSEFILE
			else
				CONFPROCESS=1
				do_putlog "Error not found [$RPMPATH/$CFILE]"
			fi

			# updtmngr install option
			if [ $TARGET_PACKAGE_UM -eq 1 ]; then
				WURL=`echo $WURL | sed -e s/\n\r//`
				if [ "$WURL" != "" ];then
					rm -f $DLSETTINGFILE.new
					while read line
					do
						KEYNAME=`echo $line | cut -c 1-10`
						if [ "$KEYNAME" = "WebsiteURL" ];then
							echo "WebsiteURL=$WURL" >>$DLSETTINGFILE.new
						else
							echo $line >>$DLSETTINGFILE.new
						fi
					done <$DLSETTINGFILE
					
					mv -f $DLSETTINGFILE.new $DLSETTINGFILE
					chmod 644 $DLSETTINGFILE
				else
					CONFPROCESS=1
					do_putlog "Error WebsiteURL empty."
				fi
			fi
		else
			CONFPROCESS=1
			do_putlog "Error not found conf file. $INSTCONF"
		fi
	else
		rm -f /opt/hitachi/snv/cli/license*
	fi

	if [ $CONFPROCESS -ne 0 ]; then
		SNV_RESULT=0
	else
		SNV_RESULT=1
	fi
}

#=<private>======================================================
# Start installation.
#  @param $1 [OEM file name]
#===============================================================
function do_install() {
	if [ $# -eq 1 ]; then
		INSTCONF=$1
	else
		INSTCONF=$RPMPATH/$INSTCONF
	fi

	is_new_installation
	if [ $SNV_RESULT -eq $SNV_FALSE ]; then
		do_putlog "update install process."

		is_old_snv_installed
		if [ $SNV_RESULT -eq $SNV_FALSE ]; then
			check_update_install
		fi

		check_already_installed
		check_rollback_install
	fi

	check_rpmfiles_exist
	do_service_stop

	if [ $INSTALLED_OLDSNV_MAJOR_VER -ne 0 ]; then
		# V2.1 or V3.0 to New Version
		do_backup_files
	fi
	
	if [ $TARGET_PACKAGE_CLI -eq 1 ]; then
		do_rpm_install $CLI_PACKAGE_NAME $SNV_CLI_VERSION
	fi

	if [ $TARGET_PACKAGE_UM -eq 1 ]; then
		do_rpm_install $UM_PACKAGE_NAME $SNV_UM_VERSION
	fi

	if [ $TARGET_PACKAGE_PCL -eq 1 ]; then
		do_rpm_install $PCL_PACKAGE_NAME $SNV_PCL_VERSION
	fi

	if [ $INSTALLED_OLDSNV_MAJOR_VER -ne 0 ]; then
		do_restore_files
	fi

	do_setup_config
	if [ $SNV_RESULT -eq $SNV_TRUE ]; then
		do_service_start
		echo -e 'The installation was successful.\n(Please eject the CDROM.)'
	else
		do_service_start
		echo "The installation was successful.(Please eject the CDROM.)"
		echo "However, it is necessary to set it again by the customer "
		echo "because it was not able to migrate it."
	fi
	do_putlog "The installation was successful."
}

#=<private>======================================================
# Start OEM installation.
#  @param $1 [OEM file]
#===============================================================
function do_install_f() {
	FILE=$1
	if [ ! -e $FILE ] ; then
		do_usage
	fi
	do_install $FILE
}

#=<private>======================================================
# Start uninstallation.
#===============================================================
function do_uninstall() {
	#env check
	if [ $INSTALLED_CLI_MAJOR_VER -eq 0 -a $INSTALLED_UM_MAJOR_VER -eq 0 -a \
		 $INSTALLED_PCL_MAJOR_VER -eq 0 -a $INSTALLED_OLDSNV_MAJOR_VER -eq 0 ] ; then
		#not installed
		echo -e 'The uninstallation failed.\n(Please eject the CDROM.)'
		do_putlog "snvcli and snvupdtmngr not found in this system."
		exit 1
	fi
	
	if [ $INSTALLED_OLDSNV_MAJOR_VER -ne 0 ]; then
		TARGET_PACKAGE_CLI=1
		TARGET_PACKAGE_UM=1
	fi

	if [ $TARGET_PACKAGE_CLI -eq 0 -a $TARGET_PACKAGE_UM -eq 1 -a $INSTALLED_UM_MAJOR_VER -eq 0 ] ; then
		#not installed
		echo -e 'The uninstallation failed.\n(Please eject the CDROM.)'
		do_putlog "snvupdtmngr not found in this system."
		exit 1
	fi

	if [ $TARGET_PACKAGE_CLI -eq 0 -a $TARGET_PACKAGE_PCL -eq 1 -a $INSTALLED_PCL_MAJOR_VER -eq 0 ] ; then
		#not installed
		echo -e 'The uninstallation failed.\n(Please eject the CDROM.)'
		do_putlog "snvpowermonitor not found in this system."
		exit 1
	fi
	
	if [ $TARGET_PACKAGE_CLI -eq 1 -a $TARGET_PACKAGE_UM -eq 0 -a $INSTALLED_UM_MAJOR_VER -ne 0 ] ; then
		#fail uninstall snvcli only
		echo -e 'The uninstallation failed.\n(Please eject the CDROM.)'
		do_putlog "can't uninstall only snvcli. updtmngr is installed."
		exit 1
	fi
	
	if [ $TARGET_PACKAGE_CLI -eq 1 -a $TARGET_PACKAGE_PCL -eq 0 -a $INSTALLED_PCL_MAJOR_VER -ne 0 ] ; then
		#fail uninstall snvcli only
		echo -e 'The uninstallation failed.\n(Please eject the CDROM.)'
		do_putlog "can't uninstall only snvcli. snvpowermonitor is installed."
		exit 1
	fi
	
	do_service_stop

	if [ $TARGET_PACKAGE_UM -eq 1 -a $INSTALLED_UM_MAJOR_VER -ne 0 ] ; then
		rpm -e --allmatches $UM_PACKAGE_NAME >>$SNV_LOGFILE 2>>$SNV_LOGFILE
		if [ $? -ne 0 ] ; then
			echo -e 'The uninstallation failed.\n(Please eject the CDROM.)'
			do_putlog "snvupdtmngr uninstallation failed."
			exit 1
		fi
	fi

	if [ $TARGET_PACKAGE_PCL -eq 1 -a $INSTALLED_PCL_MAJOR_VER -ne 0 ] ; then
		rpm -e --allmatches $PCL_PACKAGE_NAME >>$SNV_LOGFILE 2>>$SNV_LOGFILE
		if [ $? -ne 0 ] ; then
			echo -e 'The uninstallation failed.\n(Please eject the CDROM.)'
			do_putlog "snvpowermonitor uninstallation failed."
			exit 1
		fi
	fi

	if [ $TARGET_PACKAGE_CLI -eq 1 -a $INSTALLED_CLI_MAJOR_VER -ne 0 ] ; then
		rpm -e --allmatches $CLI_PACKAGE_NAME >>$SNV_LOGFILE 2>>$SNV_LOGFILE
		if [ $? -ne 0 ] ; then
			echo -e 'The uninstallation failed.\n(Please eject the CDROM.)'
			do_putlog "snvcli uninstallation failed."
			exit 1
		fi
		
		rm -rf $SNVPATH
	fi

	if [ $TARGET_PACKAGE_CLI -eq 1 -a $TARGET_PACKAGE_UM -eq 1 -a $INSTALLED_OLDSNV_MAJOR_VER -ne 0 ] ; then
		rpm -e --allmatches $OLDSNV_PACKAGE_NAME >>$SNV_LOGFILE 2>>$SNV_LOGFILE
		if [ $? -ne 0 ] ; then
			echo -e 'The uninstallation failed.\n(Please eject the CDROM.)'
			do_putlog "oldsnv uninstallation failed."
			exit 1
		fi
		
		rm -rf $SNVPATH
	fi

	echo -e 'The uninstallation was successful.\n(Please eject the CDROM.)'
	do_putlog "The uninstallation from RHEL succeeded."
}

#=<private>======================================================
# Output log to a file.
#   @param $1 message
#===============================================================
function do_putlog() {
	DATETIME=`date`
	echo "$DATETIME $1" >> $SNV_LOGFILE 2>/dev/null
}

#=<private>======================================================
# Make a new log file, and output a basic information.
#===============================================================
#function do_newlog() {
#	echo `date` >> $SNV_LOGFILE 2>/dev/null
#	echo "this installer version:$SNV_VERSION" >> $SNV_LOGFILE 2>&1
#	echo "$CLI_PACKAGE_NAME version:$CLI_VERSION" >> $SNV_LOGFILE 2>&1
#	echo "$UM_PACKAGE_NAME version:$UM_VERSION" >> $SNV_LOGFILE 2>&1
#	echo "$PCL_PACKAGE_NAME version:$PCL_VERSION" >> $SNV_LOGFILE 2>&1
#}

#=<private>======================================================
# Check the OS to support.
#   (RHEL5.x or RHEL6.x)
#===============================================================
function do_checksupportos() {
	RHVER=`cat /etc/redhat-release | cut -d" " -f7 | cut -d"." -f1`
	if [ $RHVER -ne 5 -a $RHVER -ne 6 -a $RHVER -ne 7 ];then
		echo -e 'The setup failed. (Please eject the CDROM.)\nThis OS is not supported.'
		do_putlog "This OS is not supported. (ver$RHVER)"
		uname -r >> $SNV_LOGFILE
		exit 1 >/dev/null 2>&1
	fi
}

#=<private>======================================================
# Check disk free space.
#   (At least 50MB in /opt)
#===============================================================
function do_checkdiskfreespace() {
	TARGET_DIR=/opt
	if expr "`df -m ${TARGET_DIR} | tail -n 1`" : " " > /dev/null
	then
		FREEMB=`df -m ${TARGET_DIR} | tail -n 1 | awk '{ print $3 }'`
	else
		FREEMB=`df -m ${TARGET_DIR} | tail -n 1 | awk '{ print $4 }'`
	fi
	
	if [ ${FREEMB} -lt 50 ]; then
		echo -e "The installation failed.Because there is not enough space for the installation."
		echo -e "Please confirm the free space to install"
		do_putlog "Disk free space is too small. (${FREEMB}MB)"
		exit 1 >/dev/null 2>&1
	fi
}

#=<private>======================================================
# Check the model.
#===============================================================
function do_checksupportmodel() {
	PRODUCT_NAME=`/usr/sbin/dmidecode --type system | grep "Product Name"`
	echo "$PRODUCT_NAME" | grep -q "BladeSymphony"
	if [ $? -eq 0 ] ;then
		##support model
		do_putlog "kokunai:$PRODUCT_NAME"
		return 0
	fi
	echo "$PRODUCT_NAME" | grep -q "HA8000"
	if [ $? -eq 0 ] ;then
		##support model
		do_putlog "kokunai:$PRODUCT_NAME"
		return 0
	fi
	echo "$PRODUCT_NAME" | grep -q "Compute Blade"
	if [ $? -eq 0 ] ;then
		##support model
		do_putlog "kaigai:$PRODUCT_NAME"
		return 0
	fi
	echo "$PRODUCT_NAME" | grep -q "Compute Rack"
	if [ $? -eq 0 ] ;then
		##support model
		do_putlog "kaigai:$PRODUCT_NAME"
		return 0
	fi
	
	echo -e 'This machine is un-support model.'
	do_putlog "This machine is un-support model. ($PRODUCT_NAME)"
	exit 1 >/dev/null 2>&1
}

#=<private>======================================================
# Check the powermonitor possible to install.
#===============================================================
function do_check_install_powermonitor() {
	PRODUCT_VER=`/usr/sbin/dmidecode --type system | grep "Version:" | cut -d" " -f2`
	if [[ "$PRODUCT_VER" =~ 002.R2.70. ]]; then
		do_putlog "snvpowermonitor is possible to install (product version: $PRODUCT_VER)"
		TARGET_PACKAGE_PCL=1
	elif [[ "$PRODUCT_VER" =~ 0020T2060. ]]; then
		do_putlog "snvpowermonitor is possible to install (product version: $PRODUCT_VER)"
		TARGET_PACKAGE_PCL=1
	elif [[ "$PRODUCT_VER" =~ 0010R4450. ]]; then
		do_putlog "snvpowermonitor is possible to install (product version: $PRODUCT_VER)"
		TARGET_PACKAGE_PCL=1
	elif [[ "$PRODUCT_VER" =~ 002.R2.71. ]]; then
		do_putlog "snvpowermonitor is possible to install (product version: $PRODUCT_VER)"
		TARGET_PACKAGE_PCL=1
	elif [[ "$PRODUCT_VER" =~ 002.R1.70. ]]; then
		do_putlog "snvpowermonitor is possible to install (product version: $PRODUCT_VER)"
		TARGET_PACKAGE_PCL=1
	elif [[ "$PRODUCT_VER" =~ 0020T1070. ]]; then
		do_putlog "snvpowermonitor is possible to install (product version: $PRODUCT_VER)"
		TARGET_PACKAGE_PCL=1
	elif [[ "$PRODUCT_VER" =~ 0020T2061. ]]; then
		do_putlog "snvpowermonitor is possible to install (product version: $PRODUCT_VER)"
		TARGET_PACKAGE_PCL=1
	elif [[ "$PRODUCT_VER" =~ 0010R4451. ]]; then
		do_putlog "snvpowermonitor is possible to install (product version: $PRODUCT_VER)"
		TARGET_PACKAGE_PCL=1
	elif [[ "$PRODUCT_VER" =~ 0020R1171. ]]; then
		do_putlog "snvpowermonitor is possible to install (product version: $PRODUCT_VER)"
		TARGET_PACKAGE_PCL=1
	elif [[ "$PRODUCT_VER" =~ 0020T1071. ]]; then
		do_putlog "snvpowermonitor is possible to install (product version: $PRODUCT_VER)"
		TARGET_PACKAGE_PCL=1
	else
		do_putlog "snvpowermonitor is not targeted for installation (product version: $PRODUCT_VER)"
		TARGET_PACKAGE_PCL=0
	fi
}

#=<private>======================================================
# Check the multiplex starting.
#===============================================================
function exit_if_multi_executed() {
	mkdir -p $LOGDIR >/dev/null 2>&1
	ps aux > /var/log/snv/snvinst.$$
	exec_count=`grep $CMDNAME /var/log/snv/snvinst.$$ | grep -v 'sudo' | wc -l`
	rm -f /var/log/snv/snvinst.$$ > /dev/null 2>&1
	if [ $exec_count -gt 1 ]; then
		echo -e 'The installation failed. (There are multiple installation processes.)'
		exit 1 >/dev/null 2>&1
	fi
}

#=<public>======================================================
# Display a version of this installer.
#===============================================================
function show_installer_ver() {
	echo "$CMDNAME Version = $SNV_VERSION"
}

#=<public>======================================================
# Display a version of packages.
#===============================================================
function do_showinfo() {
	show_installer_ver
	show_package_ver $OLDSNV_PACKAGE_NAME
	show_package_ver $CLI_PACKAGE_NAME
	show_package_ver $UM_PACKAGE_NAME
	show_package_ver $PCL_PACKAGE_NAME
}

#=<public>======================================================
# Install.
#===============================================================
function do_proc_install() {
	if [ $1 -eq 0 ]; then
		INS_FILE=""
		INS_PKG=$2
		INS_FORCE=$3
	else
		INS_FILE=$2
		INS_PKG=$3
		INS_FORCE=$4
	fi

	exit_if_multi_executed
	do_putlog "start install."
	if [ "$INS_PKG" = "snvcli" -a "$IS_DOMESTIC" = "true" ]; then
		do_putlog "target $INS_PKG."
		set_target_package 1 0 0
		do_checksupportos
		if [ "$INS_FORCE" != "-force" ]; then
			do_checksupportmodel
		fi
	elif [ "$INS_PKG" = "updtmngr" -a "$IS_DOMESTIC" = "true" ]; then
		do_putlog "target $INS_PKG."
		set_target_package 0 1 0
		do_checksupportos
		if [ "$INS_FORCE" != "-force" ]; then
			do_checksupportmodel
		fi
	elif [ "$INS_PKG" = "snvpowermonitor" -a "$IS_DOMESTIC" = "true" ]; then
		do_putlog "target $INS_PKG."
		set_target_package 0 0 1
		if [ "$INS_FORCE" != "-force" ]; then
			do_check_install_powermonitor
			if [ $TARGET_PACKAGE_PCL -eq 0 ] ; then
				do_usage
			fi
		fi
		do_checksupportos
		if [ "$INS_FORCE" != "-force" ]; then
			do_checksupportmodel
		fi
	elif [ "$INS_PKG" = "-force" ]; then
		do_putlog "force install."
		set_target_package 1 1 1
		do_checksupportos
	elif [ "$INS_PKG" != "" ]; then
		do_putlog "unknown paramater $INS_PKG"
		do_usage
	else
		do_putlog "no selected package."
		NON_SELECTED_PACKAGE=1
		if [ "$IS_DOMESTIC" = "true" ]; then
			set_target_package 1 1 1
		else
			set_target_package 1 0 1
		fi
		do_check_install_powermonitor
		do_checksupportos
		do_checksupportmodel
	fi
	get_installed_package_versions
	do_checkdiskfreespace
	if [ -z $INS_FILE ]; then
		do_install
	else
		do_install_f $INS_FILE
	fi
}

#=<public>======================================================
# Uninstall.
#===============================================================
function do_proc_uninstall() {
	INS_PKG=$1

	exit_if_multi_executed
	do_putlog "start uninstall."
	do_checksupportos
	if [ "$INS_PKG" = "snvcli" -a "$IS_DOMESTIC" = "true" ]; then
		do_putlog "target $2."
		set_target_package 1 0 0
	elif [ "$INS_PKG" = "updtmngr" -a "$IS_DOMESTIC" = "true" ]; then
		do_putlog "target $INS_PKG."
		set_target_package 0 1 0
	elif [ "$INS_PKG" = "snvpowermonitor" -a "$IS_DOMESTIC" = "true" ]; then
		do_putlog "target $INS_PKG."
		set_target_package 0 0 1
	elif [ "$INS_PKG" = "-force" ]; then
		do_putlog "target $INS_PKG."
		set_target_package 1 1 1
	elif [ "$INS_PKG" != "" ]; then
		do_putlog "unknown paramater $INS_PKG"
		do_usage
	fi
	get_installed_package_versions
	do_uninstall
}


#=<public>======================================================
# Check domestic server type.
#===============================================================
function do_check_domestic() {

	PRODUCT_NAME=`dmidecode --type system | grep "Product Name"`
	echo "$PRODUCT_NAME" | grep -q "Compute Blade"
	if [ $? -eq 0 ] ;then
		IS_DOMESTIC="false"
	else
		echo "$PRODUCT_NAME" | grep -q "Compute Rack"
		if [ $? -eq 0 ] ;then
			IS_DOMESTIC="false"
		fi
	fi

}

#=<public>======================================================
#=====<< Main proc. >>==========================================
#===============================================================
ARGC=$#
CMD=$1

USERID=`id | sed 's/uid=\([0-9]*\)(.*/\1/'`
if [ $USERID -ne 0 ];then
	echo "Permission denied."
    exit 1 >/dev/null 2>&1
fi 

do_check_domestic

case "$CMD" in
"-i") do_proc_install 0 $2 $3;;
"-o") do_proc_install 1 $2 $3 $4;;
"-e") do_proc_uninstall $2;;
"-l") do_showinfo;;
"--version") show_installer_ver;;
*) do_usage;;
esac

