#!/bin/sh
#################################################################
# ssh engine for Provider of transfer file.
# -- Excecute the command on the ESXi host by ssh
#
# [Parameters]
#  $1 : HOSTNAME  Connection host name or IP address.
#  $2 : USERNAME  User name.
#  $3 : PASSWORD  Password.
#  $4 : CMDEXEC   Execute command.
#
# [history]
#  2017.08.30 Add option(-o StrictHostKeyChecking=no
#              -o UserKnownHostsFile=/dev/null)
#
#################################################################
if [ $# -lt 4 ] ; then
	exit 1
fi
HOSTNAME=$1
USERNAME=$2
PASSWORD=$3
CMDEXEC=$4
DEBUGOPT=
if [ $# -eq 5 ]; then
	DEBUGOPT=$5
fi

expect ${DEBUGOPT} -c "
set timeout 20
set ConEsxi 0
set pwcount 2

log_user 0
spawn ssh -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null ${USERNAME}@${HOSTNAME}
expect_after eof {
	if {\$ConEsxi == 0} {
		send_user \"connection to ${HOSTNAME} failed\n\"
		exit 5
	}
}

expect {
	\"\[Pp]assword:\" {
		send \"${PASSWORD}\n\"
		set pwcount [expr \$pwcount-1]
		if {\$pwcount == 0} {
			send_user \"invalid password or account\n\"
			set ConEsxi 0
			exit 6
		}
		set ConEsxi 1
		exp_continue
	} \"${USERNAME}@${HOSTNAME}'s \[Pp]assword:\" {
		send \"${PASSWORD}\n\"
		set pwcount [expr \$pwcount-1]
		if {\$pwcount == 0} {
			send_user \"invalid password or account\n\"
			set ConEsxi 0
			exit 6
		}
		set ConEsxi 1
		exp_continue
	} incorrect {
		send_user \"invalid password or account\n\"
		set ConEsxi 0
		exit 3
	} \"scp: ${SRCPATH}: No such file or directory\" {
		send_user \"file or directory not found\n\"
		exit 2
	} \"${DSTPATH}: No such file or directory\" {
		send_user \"file or directory not found\n\"
		exit 2
	} timeout {
		send_user \"connection to ${HOSTNAME} timed out\n\"
		exit 4
	}
}
"
RESULT=$?

if [ ! -z ${DEBUGOPT} ]; then
	echo "RESULT=${RESULT}"
fi
exit ${RESULT}
