#!/bin/bash

#===============================================================
# Version definitions of this package.
#===============================================================
# snvsetup.sh info
OS_KIND=2
MAJOR_VER=4
RELEASE_VER=8.0
export SNV_VERSION=${OS_KIND}.${MAJOR_VER}.${RELEASE_VER}

# snvcli <RHEL> (snvcli + providers) package info
export SNV_CLI_MAJOR_VER=4
export SNV_CLI_RELEASE_VER=8.0
export SNV_CLI_VERSION=${OS_KIND}.${SNV_CLI_MAJOR_VER}-${SNV_CLI_RELEASE_VER}

# snvcli <VRHEL> (snvcli + providers) package info  *** new ***
export SNV_VCLI_MAJOR_VER=4
export SNV_VCLI_RELEASE_VER=8.0
export SNV_VCLI_VERSION=${OS_KIND}.${SNV_VCLI_MAJOR_VER}-${SNV_VCLI_RELEASE_VER}

# snvupdtmngr (update manager) package info
export SNV_UM_MAJOR_VER=4
export SNV_UM_RELEASE_VER=7.0
export SNV_UM_VERSION=${OS_KIND}.${SNV_UM_MAJOR_VER}-${SNV_UM_RELEASE_VER}

# snvpowermonitor package info
export SNV_PCL_MAJOR_VER=4
export SNV_PCL_RELEASE_VER=7.0
export SNV_PCL_VERSION=${OS_KIND}.${SNV_PCL_MAJOR_VER}-${SNV_PCL_RELEASE_VER}

# hrn package info (Guest RHEL Only)
export SNV_HRN_MAJOR_VER=16
export SNV_HRN_RELEASE_VER=0.0
export SNV_HRN_VERSION=${OS_KIND}.${SNV_HRN_MAJOR_VER}-${SNV_HRN_RELEASE_VER}

SETUPSHPATH=$(cd $(dirname $0); pwd)
#---------------------------------------------------------------
# Create new log file.
#---------------------------------------------------------------
export OS_KIND
export SNV_PARENT_SHELL=`basename $0`
export SNV_LOGDIR=/var/log/snv
export SNV_LOGFILE=$SNV_LOGDIR/snv_setup.log

rm -f $SNV_LOGFILE 2>&1
mkdir -p $SNV_LOGDIR >/dev/null 2>&1
echo -n "" >$SNV_LOGFILE 2>&1

#---------------------------------------------------------------
# Get the CPU architecture.
#---------------------------------------------------------------
SNV_CPU_ARCH=
uname -i | grep -q "x86_64"
if [ $? -eq 0 ] ; then
	SNV_CPU_ARCH="x86_64"
else
	SNV_CPU_ARCH="i386"
fi
export SNV_CPU_ARCH

#---------------------------------------------------------------
# Get the Platform type and more.
#---------------------------------------------------------------
RHEL_INFO=/etc/redhat-release
VMA_INFO=/etc/vma-release

IS_GUESTOS=100
which virt-what > /dev/null 2>&1
if [ $? -eq 0 ]; then
	IS_GUESTOS=1
	VMMACHINE=`virt-what`
	if [ ${#VMMACHINE} -ne 0 ]; then
		echo "`date` virt-what: ${VMMACHINE}" >> $SNV_LOGFILE 2>&1
		if [ $VMMACHINE = "vmware" ]; then
			IS_GUESTOS=0
		fi
	fi
fi

# for debug option.
echo "$@" | grep -e "-force" > /dev/null 2>&1
if [ $? -eq 0 ]; then
	IS_GUESTOS=1
fi
echo "`date` GUESTOS=${IS_GUESTOS}" >> $SNV_LOGFILE 2>&1

#---------------------------------------------------------------
# Execute sub shell.
#---------------------------------------------------------------
if [ -e $RHEL_INFO -a $IS_GUESTOS -eq 0 ]; then
	#** Guest RHEL **
	/bin/sh $SETUPSHPATH/velrpms/velsetup.sh "$@"
	RESULT=$?

elif [ -e $RHEL_INFO -a $IS_GUESTOS -eq 1 ]; then
	#** Real RHEL **
	/bin/sh $SETUPSHPATH/elrpms/elsetup.sh "$@"
	RESULT=$?

else
	echo -e 'The setup failed. (Please eject the CDROM.)\nThis OS is not supported.'
	date >> $SNV_LOGFILE
	echo "This OS is not supported. " >> $SNV_LOGFILE

	dmidecode -t system | grep -A 3 'System Information' >> $SNV_LOGFILE

	if [ ! -e $RHEL_INFO ]; then
		echo " - $RHEL_INFO not found" >> $SNV_LOGFILE
	fi
	RESULT=1
fi

exit $RESULT

